@@ -68,13 +68,38 @@ TEXT = "text"
 """
 
 OFF = 0
-"""No database profiling."""
-SLOW_ONLY = 1
-"""Only profile slow operations."""
-ALL = 2
-"""Profile all operations."""
+"""**DEPRECATED** - No database profiling.
 
-version_tuple = (3, 11, 3)
+**DEPRECATED** - :attr:`OFF` is deprecated and will be removed in PyMongo 4.0.
+Instead, specify this profiling level using the numeric value ``0``.
+See https://docs.mongodb.com/manual/tutorial/manage-the-database-profiler
+
+.. versionchanged:: 3.12
+   Deprecated
+"""
+SLOW_ONLY = 1
+"""**DEPRECATED** - Only profile slow operations.
+
+**DEPRECATED** - :attr:`SLOW_ONLY` is deprecated and will be removed in
+PyMongo 4.0. Instead, specify this profiling level using the numeric
+value ``1``.
+See https://docs.mongodb.com/manual/tutorial/manage-the-database-profiler
+
+.. versionchanged:: 3.12
+   Deprecated
+"""
+ALL = 2
+"""**DEPRECATED** - Profile all operations.
+
+**DEPRECATED** - :attr:`ALL` is deprecated and will be removed in PyMongo 4.0.
+Instead, specify this profiling level using the numeric value ``2``.
+See https://docs.mongodb.com/manual/tutorial/manage-the-database-profiler
+
+.. versionchanged:: 3.12
+   Deprecated
+"""
+
+version_tuple = (3, 12, 2)
 
 def get_version_string():
     if isinstance(version_tuple[-1], str):
